@@ -1,4 +1,5 @@
 use crate::{
+    data_type::DataType,
     definitions::{
         N_BINOPS_OF_DEEPEX_ON_STACK, N_NODES_ON_STACK, N_UNARYOPS_OF_DEEPEX_ON_STACK,
         N_VARS_ON_STACK,
@@ -6,10 +7,8 @@ use crate::{
     expression::deep_details::{
         self, prioritized_indices, BinOpsWithReprsBuf, UnaryOpWithReprsBuf,
     },
-    operators::{BinOp, DefaultOpsFactory, MakeOperators, UnaryOp},
-    parser,
-    data_type::DataType,
-    ExError, ExResult, Operator,
+    operators::{BinOp, FloatOpsFactory, MakeOperators, UnaryOp},
+    parser, ExError, ExResult, Operator,
 };
 use num::Float;
 use regex::Regex;
@@ -129,13 +128,10 @@ pub struct DeepEx<'a, T: Clone + Debug> {
     var_names: SmallVec<[&'a str; N_VARS_ON_STACK]>,
 }
 
-fn lift_nodes<'a, T: Clone + Debug>(deepex: &mut DeepEx<'a, T>) {
+fn lift_nodes<T: Clone + Debug>(deepex: &mut DeepEx<T>) {
     if deepex.nodes.len() == 1 && deepex.unary_op.op.len() == 0 {
-        match &deepex.nodes[0] {
-            DeepNode::Expr(e) => {
-                *deepex = e.clone();
-            }
-            _ => (),
+        if let DeepNode::Expr(e) = &deepex.nodes[0] {
+            *deepex = e.clone();
         }
     } else {
         for node in &mut deepex.nodes {
@@ -177,7 +173,8 @@ impl<'a, T: Clone + Debug> DeepEx<'a, T> {
             let node_2 = &self.nodes[num_idx + 1];
             if let (DeepNode::Num(num_1), DeepNode::Num(num_2)) = (node_1, node_2) {
                 if !(already_declined[num_idx] || already_declined[num_idx + 1]) {
-                    let bin_op_result = (self.bin_ops.ops[bin_op_idx].apply)(num_1.clone(), num_2.clone());
+                    let bin_op_result =
+                        (self.bin_ops.ops[bin_op_idx].apply)(num_1.clone(), num_2.clone());
                     self.nodes[num_idx] = DeepNode::Num(bin_op_result);
                     self.nodes.remove(num_idx + 1);
                     already_declined.remove(num_idx + 1);
@@ -454,7 +451,7 @@ impl<'a, T: Clone + Debug> DeepEx<'a, T> {
         <T as std::str::FromStr>::Err: Debug,
         T: Float + DataType,
     {
-        let ops = DefaultOpsFactory::<T>::make();
+        let ops = FloatOpsFactory::<T>::make();
         DeepEx::from_ops(text, &ops)
     }
 
@@ -466,6 +463,19 @@ impl<'a, T: Clone + Debug> DeepEx<'a, T> {
         parse(text, ops, parser::is_numeric_text)
     }
 
+    pub fn from_regex(
+        text: &'a str,
+        ops: &[Operator<'a, T>],
+        re_number: &Regex,
+    ) -> ExResult<DeepEx<'a, T>>
+    where
+        <T as std::str::FromStr>::Err: Debug,
+        T: DataType,
+    {
+        let is_numeric = |text: &'a str| parser::is_numeric_regex(&re_number, text);
+        parse(text, ops, is_numeric)
+    }
+
     pub fn from_pattern(
         text: &'a str,
         ops: &[Operator<'a, T>],
@@ -475,8 +485,7 @@ impl<'a, T: Clone + Debug> DeepEx<'a, T> {
         <T as std::str::FromStr>::Err: Debug,
         T: DataType,
     {
-        let beginning_num_re_pattern = format!("^({})", number_regex_pattern);
-        let re_number = match Regex::new(beginning_num_re_pattern.as_str()) {
+        let re_number = match Regex::new(number_regex_pattern) {
             Ok(regex) => regex,
             Err(_) => {
                 return Err(ExError {
@@ -484,8 +493,7 @@ impl<'a, T: Clone + Debug> DeepEx<'a, T> {
                 })
             }
         };
-        let is_numeric = |text: &'a str| parser::is_numeric_regex(&re_number, text);
-        parse(text, ops, is_numeric)
+        Self::from_regex(text, ops, &re_number)
     }
 
     pub fn eval(&self, vars: &[T]) -> ExResult<T> {
@@ -649,7 +657,7 @@ fn test_var_name_union() {
 
 #[test]
 fn test_partial_finite() {
-    let ops = DefaultOpsFactory::<f64>::make();
+    let ops = FloatOpsFactory::<f64>::make();
     fn test<'a>(sut: &str, ops: &'a [Operator<'a, f64>], range: Range<f64>) {
         let dut = DeepEx::<f64>::from_str_float(sut).unwrap();
         let n_vars = dut.n_vars();
@@ -715,7 +723,7 @@ fn test_var_names() {
 
 #[test]
 fn test_deep_compile() {
-    let ops = DefaultOpsFactory::make();
+    let ops = FloatOpsFactory::make();
     let nodes = vec![DeepNode::Num(4.5), DeepNode::Num(0.5), DeepNode::Num(1.4)];
     let bin_ops = BinOpsWithReprs {
         reprs: smallvec![ops[1].repr(), ops[3].repr()],
@@ -768,7 +776,7 @@ fn test_deep_compile_2() {
     let deepex = DeepEx::<f64>::from_str_float("1+(((a+x^2*x^2)))").unwrap();
     println!("{}", deepex);
     assert_eq!(format!("{}", deepex), "1.0+({a}+{x}^2.0*{x}^2.0)");
-    let mut ddeepex = partial_deepex(1, deepex, &DefaultOpsFactory::make()).unwrap();
+    let mut ddeepex = partial_deepex(1, deepex, &FloatOpsFactory::make()).unwrap();
     ddeepex.compile();
     println!("{}", ddeepex);
     assert_eq!(
@@ -776,3 +784,25 @@ fn test_deep_compile_2() {
         "(({x}^2.0)*({x}*2.0))+(({x}*2.0)*({x}^2.0))"
     );
 }
+
+#[test]
+fn test_compile() {
+    let expr = DeepEx::<f64>::from_str_float("1.0 * 3 * 2 * x / 2 / 3").unwrap();
+    assert_float_eq_f64(expr.eval(&[2.0]).unwrap(), 2.0);
+    let expr = DeepEx::<f64>::from_str_float(
+        "x*0.2*5/4+x*2*4*1*1*1*1*1*1*1+2+3+7*sin(y)-z/sin(3.0/2/(1-x*4*1*1*1*1))",
+    )
+    .unwrap();
+    assert_eq!(
+        "{x}*0.25+{x}*8.0+5.0+7.0*sin({y})-{z}/sin(1.5/(1.0-{x}*4.0))",
+        expr.unparse_raw()
+    );
+    let expr = DeepEx::<f64>::from_str_float("x + 1 - 2").unwrap();
+    assert_float_eq_f64(expr.eval(&[0.0]).unwrap(), -1.0);
+    let expr = DeepEx::<f64>::from_str_float("x - 1 + 2").unwrap();
+    assert_float_eq_f64(expr.eval(&[0.0]).unwrap(), 1.0);
+    let expr = DeepEx::<f64>::from_str_float("x * 2 / 3").unwrap();
+    assert_float_eq_f64(expr.eval(&[2.0]).unwrap(), 4.0 / 3.0);
+    let expr = DeepEx::<f64>::from_str_float("x / 2 / 3").unwrap();
+    assert_float_eq_f64(expr.eval(&[2.0]).unwrap(), 1.0 / 3.0);
+}
