@@ -21,20 +21,21 @@ pub fn find_op<'a, T: Copy + Debug>(
     repr: &'a str,
     ops: &[Operator<'a, T>],
 ) -> Option<Operator<'a, T>> {
-    let found = ops.iter().cloned().find(|op| op.repr() == repr);
-    found.map(|op| op)
+    ops.iter().cloned().find(|op| op.repr() == repr)
 }
 
+type BinOpPartial<'a, T> = fn(
+    ValueDerivative<'a, T>,
+    ValueDerivative<'a, T>,
+    &[Operator<'a, T>],
+) -> ExResult<ValueDerivative<'a, T>>;
+
+type UnaryOpOuter<'a, T> = fn(DeepEx<'a, T>, &[Operator<'a, T>]) -> ExResult<DeepEx<'a, T>>;
+
 pub struct PartialDerivative<'a, T: Copy + Debug> {
     repr: &'a str,
-    bin_op: Option<
-        fn(
-            ValueDerivative<'a, T>,
-            ValueDerivative<'a, T>,
-            &[Operator<'a, T>],
-        ) -> ExResult<ValueDerivative<'a, T>>,
-    >,
-    unary_outer_op: Option<fn(DeepEx<'a, T>, &[Operator<'a, T>]) -> ExResult<DeepEx<'a, T>>>,
+    bin_op: Option<BinOpPartial<'a, T>>,
+    unary_outer_op: Option<UnaryOpOuter<'a, T>>,
 }
 
 fn find_as_bin_op_with_reprs<'a, T: Copy + Debug>(
@@ -292,24 +293,24 @@ fn pow<'a, T: Float + Debug>(
 }
 
 fn mul_find<'a, T: Copy + Debug>(ops: &[Operator<'a, T>]) -> ExResult<BinOpsWithReprs<'a, T>> {
-    find_as_bin_op_with_reprs("*", &ops)
+    find_as_bin_op_with_reprs("*", ops)
 }
 fn div_find<'a, T: Copy + Debug>(ops: &[Operator<'a, T>]) -> ExResult<BinOpsWithReprs<'a, T>> {
-    find_as_bin_op_with_reprs("/", &ops)
+    find_as_bin_op_with_reprs("/", ops)
 }
 fn add_find<'a, T: Copy + Debug>(ops: &[Operator<'a, T>]) -> ExResult<BinOpsWithReprs<'a, T>> {
-    find_as_bin_op_with_reprs("+", &ops)
+    find_as_bin_op_with_reprs("+", ops)
 }
 fn sub_find<'a, T: Copy + Debug>(ops: &[Operator<'a, T>]) -> ExResult<BinOpsWithReprs<'a, T>> {
-    find_as_bin_op_with_reprs("-", &ops)
+    find_as_bin_op_with_reprs("-", ops)
 }
 fn pow_find<'a, T: Copy + Debug>(ops: &[Operator<'a, T>]) -> ExResult<BinOpsWithReprs<'a, T>> {
-    find_as_bin_op_with_reprs("^", &ops)
+    find_as_bin_op_with_reprs("^", ops)
 }
 fn minus_find_unary<'a, T: Copy + Debug>(
     ops: &[Operator<'a, T>],
 ) -> ExResult<UnaryOpWithReprs<'a, T>> {
-    find_as_unary_op_with_reprs("-", &ops)
+    find_as_unary_op_with_reprs("-", ops)
 }
 
 pub fn make_partial_derivative_ops<'a, T: Float + Debug>() -> Vec<PartialDerivative<'a, T>> {
@@ -611,13 +612,13 @@ pub fn make_partial_derivative_ops<'a, T: Float + Debug>() -> Vec<PartialDerivat
 
 #[cfg(test)]
 use crate::{
-    operators::{DefaultOpsFactory, MakeOperators},
+    operators::{FloatOpsFactory, MakeOperators},
     util::assert_float_eq_f64,
 };
 
 #[test]
 fn test_partial() {
-    let ops = DefaultOpsFactory::<f64>::make();
+    let ops = FloatOpsFactory::<f64>::make();
     let dut = DeepEx::<f64>::from_str_float("z*sin(x)+cos(y)^(sin(z))").unwrap();
     let d_z = partial_deepex(2, dut.clone(), &ops).unwrap();
     assert_float_eq_f64(
@@ -643,7 +644,7 @@ fn test_partial_3_vars() {
         deepex.eval(vars).unwrap()
     }
     fn assert(s: &str, vars: &[f64], ref_vals: &[f64]) {
-        let ops = DefaultOpsFactory::<f64>::make();
+        let ops = FloatOpsFactory::<f64>::make();
         let dut = DeepEx::<f64>::from_str_float(s).unwrap();
         let d_x = partial_deepex(0, dut.clone(), &ops).unwrap();
         assert_float_eq_f64(eval(&d_x, vars), ref_vals[0]);
@@ -662,7 +663,7 @@ fn test_partial_3_vars() {
 
 #[test]
 fn test_partial_x2x() {
-    let ops = DefaultOpsFactory::<f64>::make();
+    let ops = FloatOpsFactory::<f64>::make();
     let deepex = DeepEx::<f64>::from_str_float("x * 2 * x").unwrap();
     let derivative = partial_deepex(0, deepex.clone(), &ops).unwrap();
     let result = derivative.eval(&[0.0]).unwrap();
@@ -673,7 +674,7 @@ fn test_partial_x2x() {
 
 #[test]
 fn test_partial_cos_squared() {
-    let ops = DefaultOpsFactory::<f64>::make();
+    let ops = FloatOpsFactory::<f64>::make();
     let deepex = DeepEx::<f64>::from_str_float("cos(y) ^ 2").unwrap();
     let derivative = partial_deepex(0, deepex.clone(), &ops).unwrap();
     let result = derivative.eval(&[0.0]).unwrap();
@@ -684,7 +685,7 @@ fn test_partial_cos_squared() {
 
 #[test]
 fn test_num_ops() {
-    let ops = DefaultOpsFactory::<f64>::make();
+    let ops = FloatOpsFactory::<f64>::make();
     let mul_op = find_as_bin_op_with_reprs("*", &ops).unwrap();
     fn eval<'a>(deepex: &DeepEx<'a, f64>, vars: &[f64], val: f64) {
         assert_float_eq_f64(deepex.eval(vars).unwrap(), val);
@@ -703,7 +704,7 @@ fn test_num_ops() {
 
 #[test]
 fn test_partial_combined() {
-    let ops = DefaultOpsFactory::<f64>::make();
+    let ops = FloatOpsFactory::<f64>::make();
     let deepex = DeepEx::<f64>::from_str_float("sin(x) + cos(y) ^ 2").unwrap();
     let d_y = partial_deepex(1, deepex.clone(), &ops).unwrap();
     let result = d_y.eval(&[231.431, 0.0]).unwrap();
@@ -719,7 +720,7 @@ fn test_partial_combined() {
 
 #[test]
 fn test_partial_derivative_second_var() {
-    let ops = DefaultOpsFactory::<f64>::make();
+    let ops = FloatOpsFactory::<f64>::make();
     let deepex = DeepEx::<f64>::from_str_float("sin(x) + cos(y)").unwrap();
     let derivative = partial_deepex(1, deepex.clone(), &ops).unwrap();
     let result = derivative.eval(&[231.431, 0.0]).unwrap();
@@ -730,7 +731,7 @@ fn test_partial_derivative_second_var() {
 
 #[test]
 fn test_partial_derivative_first_var() {
-    let ops = DefaultOpsFactory::<f64>::make();
+    let ops = FloatOpsFactory::<f64>::make();
 
     let deepex = DeepEx::<f64>::from_str_float("sin(x) + cos(y)").unwrap();
     let derivative = partial_deepex(0, deepex.clone(), &ops).unwrap();
@@ -744,7 +745,7 @@ fn test_partial_derivative_first_var() {
 fn test_partial_inner() {
     fn test(text: &str, vals: &[f64], ref_vals: &[f64], var_idx: usize) {
         let partial_derivative_ops = make_partial_derivative_ops::<f64>();
-        let ops = DefaultOpsFactory::<f64>::make();
+        let ops = FloatOpsFactory::<f64>::make();
         let deepex_1 = DeepEx::<f64>::from_str_float(text).unwrap();
         let deri =
             partial_derivative_inner(var_idx, deepex_1, &partial_derivative_ops, &ops).unwrap();
@@ -760,7 +761,7 @@ fn test_partial_inner() {
 fn test_partial_outer() {
     fn test(text: &str, vals: &[f64], ref_vals: &[f64]) {
         let partial_derivative_ops = make_partial_derivative_ops::<f64>();
-        let ops = DefaultOpsFactory::<f64>::make();
+        let ops = FloatOpsFactory::<f64>::make();
         let deepex_1 = DeepEx::<f64>::from_str_float(text).unwrap();
         let deepex = deepex_1.nodes()[0].clone();
 
@@ -785,7 +786,7 @@ fn test_partial_outer() {
 
 #[test]
 fn test_partial_derivative_simple() {
-    let ops = DefaultOpsFactory::<f64>::make();
+    let ops = FloatOpsFactory::<f64>::make();
 
     let deepex = DeepEx::<f64>::from_str_float("1").unwrap();
     let derivative = partial_deepex(0, deepex, &ops).unwrap();
