@@ -1,12 +1,19 @@
-use crate::{definitions::N_UNARYOPS_OF_DEEPEX_ON_STACK, ExError, ExResult};
+use crate::{ExError, ExResult, definitions::N_UNARYOPS_OF_DEEPEX_ON_STACK, format_exerr};
 use num::Float;
 use smallvec::{smallvec, SmallVec};
 use std::{fmt::Debug, marker::PhantomData};
 
-fn make_op_not_available_error<'a>(repr: &'a str) -> ExError {
-    ExError {
-        msg: format!("operator {} not available", repr),
-    }
+enum OperatorType {
+    Bin,
+    Unary
+}
+
+fn make_op_not_available_error(repr: &str, op_type: OperatorType) -> ExError {
+    let op_type_str = match op_type {
+        OperatorType::Bin => "binary",
+        OperatorType::Unary => "unary"
+    };
+    format_exerr!("{} operator {} not available", op_type_str, repr)    
 }
 
 /// Operators can be custom-defined by the library-user in terms of this struct.
@@ -23,8 +30,8 @@ pub struct Operator<'a, T: Clone> {
     constant: Option<T>,
 }
 
-fn unwrap_operator<'a, O>(wrapped_op: &'a Option<O>, repr: &str) -> ExResult<&'a O> {
-    wrapped_op.as_ref().ok_or(make_op_not_available_error(repr))
+fn unwrap_operator<'a, O>(wrapped_op: &'a Option<O>, repr: &str, op_type: OperatorType) -> ExResult<&'a O> {
+    wrapped_op.as_ref().ok_or_else(||make_op_not_available_error(repr, op_type))
 }
 
 impl<'a, T: Clone> Operator<'a, T> {
@@ -65,7 +72,7 @@ impl<'a, T: Clone> Operator<'a, T> {
         Operator::new(repr, None, Some(unary_op), None)
     }
     /// Creates an operator that is either unary or binary based on its positioning in the string to be parsed.
-    /// For instance, `-` as defined in [`DefaultOpsFactory`](DefaultOpsFactory) is unary in `-x` and binary
+    /// For instance, `-` as defined in [`FloatOpsFactory`](FloatOpsFactory) is unary in `-x` and binary
     /// in `2-x`.
     pub fn make_bin_unary(
         repr: &'a str,
@@ -80,11 +87,11 @@ impl<'a, T: Clone> Operator<'a, T> {
     }
 
     pub fn bin(&self) -> ExResult<BinOp<T>> {
-        let op = unwrap_operator(&self.bin_op, self.repr)?;
+        let op = unwrap_operator(&self.bin_op, self.repr, OperatorType::Bin)?;
         Ok(op.clone())
     }
     pub fn unary(&self) -> ExResult<fn(T) -> T> {
-        Ok(*unwrap_operator(&self.unary_op, self.repr)?)
+        Ok(*unwrap_operator(&self.unary_op, self.repr, OperatorType::Unary)?)
     }
     pub fn repr(&self) -> &'a str {
         self.repr
@@ -128,7 +135,7 @@ impl<T> UnaryOp<T> {
             .funcs_to_be_composed
             .iter()
             .chain(self.funcs_to_be_composed.iter())
-            .map(|f| *f)
+            .copied()
             .collect::<SmallVec<_>>();
     }
 
@@ -201,12 +208,45 @@ pub trait MakeOperators<T: Clone>: Clone {
 }
 
 /// Factory of default operators for floating point values.
+/// 
+/// |representation|description|
+/// |--------------|-----------|
+/// |`^`| power |
+/// |`*`| product |
+/// |`/`| division |
+/// |`+`| addition as binary or identity as unary operator|
+/// |`-`| subtraction as binary or inverting the sign as unary operator |
+/// |`abs`| absolute value |
+/// |`signum`| signum |
+/// |`sin`| sine |
+/// |`cos`| cosine |
+/// |`tan`| tangent |
+/// |`asin`| inverse sine |
+/// |`acos`| inverse cosine |
+/// |`atan`| inverse tangent |
+/// |`sinh`| hyperbolic sine |
+/// |`cosh`| hyperbolic cosine |
+/// |`tanh`| hyperbolic tangent |
+/// |`floor`| largest integer less than or equal to a number |
+/// |`ceil`| smallest integer greater than or equal to a number |
+/// |`trunc`| integer part of a number |
+/// |`fract`| fractional part of a number |
+/// |`exp`| exponential functionn |
+/// |`sqrt`| square root |
+/// |`cbrt`| cube root |
+/// |`log`| natural logarithm  |
+/// |`log2`| logarithm with basis 2  |
+/// |`PI`| constant π  |
+/// |`π`| second representations of constant π  |
+/// |`E`| Euler's number |
+/// 
+/// 
 #[derive(Clone, Eq, PartialEq, Ord, PartialOrd, Debug)]
-pub struct DefaultOpsFactory<T: Float> {
+pub struct FloatOpsFactory<T: Float> {
     dummy: PhantomData<T>,
 }
 
-impl<T: Float> MakeOperators<T> for DefaultOpsFactory<T> {
+impl<T: Float> MakeOperators<T> for FloatOpsFactory<T> {
     /// Returns the default operators.
     fn make<'a>() -> Vec<Operator<'a, T>> {
         vec![
@@ -252,6 +292,7 @@ impl<T: Float> MakeOperators<T> for DefaultOpsFactory<T> {
                 },
                 |a| -a,
             ),
+            Operator::make_unary("abs", |a| a.abs()),
             Operator::make_unary("signum", |a| a.signum()),
             Operator::make_unary("sin", |a| a.sin()),
             Operator::make_unary("cos", |a| a.cos()),
@@ -263,11 +304,13 @@ impl<T: Float> MakeOperators<T> for DefaultOpsFactory<T> {
             Operator::make_unary("cosh", |a| a.cosh()),
             Operator::make_unary("tanh", |a| a.tanh()),
             Operator::make_unary("floor", |a| a.floor()),
+            Operator::make_unary("round", |a| a.round()),
             Operator::make_unary("ceil", |a| a.ceil()),
             Operator::make_unary("trunc", |a| a.trunc()),
             Operator::make_unary("fract", |a| a.fract()),
             Operator::make_unary("exp", |a| a.exp()),
             Operator::make_unary("sqrt", |a| a.sqrt()),
+            Operator::make_unary("cbrt", |a| a.cbrt()),
             Operator::make_unary("log", |a| a.ln()),
             Operator::make_unary("log2", |a| a.log2()),
             Operator::make_constant("PI", T::from(std::f64::consts::PI).unwrap()),
