@@ -1,5 +1,6 @@
-use crate::definitions::N_NODES_ON_STACK;
 use crate::data_type::DataType;
+use crate::definitions::N_NODES_ON_STACK;
+use crate::format_exerr;
 use crate::{operators::Operator, ExError, ExResult};
 use lazy_static::lazy_static;
 use regex::Regex;
@@ -20,7 +21,7 @@ pub enum ParsedToken<'a, T: DataType> {
     Var(&'a str),
 }
 
-pub fn is_numeric_text<'a>(text: &'a str) -> Option<&'a str> {
+pub fn is_numeric_text(text: &str) -> Option<&str> {
     let mut n_dots = 0;
     let n_num_chars = text
         .chars()
@@ -92,19 +93,13 @@ where
     let find_ops = |byte_offset: usize| {
         ops.iter().find(|op| {
             let range_end = byte_offset + op.repr().len();
-            if let Some(maybe_op) = &text.get(byte_offset..range_end) {
-                if op.repr() != *maybe_op {
-                    false
-                } else if !op.has_bin()
-                    && range_end < text.len()
-                    && RE_VAR_NAME_EXACT.is_match(
-                        &text[byte_offset..range_end + next_char_boundary(&text, range_end)],
-                    )
-                {
-                    false
-                } else {
-                    true
-                }
+            if let Some(maybe_op) = text.get(byte_offset..range_end) {
+                op.repr() == maybe_op
+                    && (op.has_bin()
+                        || range_end >= text.len()
+                        || !RE_VAR_NAME_EXACT.is_match(
+                            &text[byte_offset..range_end + next_char_boundary(text, range_end)],
+                        ))
             } else {
                 false
             }
@@ -113,7 +108,7 @@ where
     let mut res: SmallVec<[_; N_NODES_ON_STACK]> = SmallVec::new();
     let mut cur_byte_offset = 0usize;
     for (i, c) in text.char_indices() {
-        if c == ' ' {
+        if c == ' ' && i == cur_byte_offset {
             cur_byte_offset += 1;
         } else if i == cur_byte_offset && cur_byte_offset < text.len() {
             let text_rest = &text[cur_byte_offset..];
@@ -131,9 +126,7 @@ where
                     .map(|c| c.len_utf8())
                     .sum();
                 let var_name = &text_rest[1..n_count];
-                let n_spaces = var_name.chars().filter(|c| *c == ' ').count();
-                // we need to subtract spaces from the offset, since they are added in the first if again.
-                cur_byte_offset += n_count + 1 - n_spaces;
+                cur_byte_offset += n_count + 1;
                 ParsedToken::<T>::Var(var_name)
             } else if let Some(num_str) = is_numeric(text_rest) {
                 let n_bytes = num_str.len();
@@ -148,7 +141,7 @@ where
                     Some(constant) => ParsedToken::<T>::Num(constant),
                     None => ParsedToken::<T>::Op((*op).clone()),
                 }
-            } else if let Some(var_str) = RE_VAR_NAME.find(&text_rest) {
+            } else if let Some(var_str) = RE_VAR_NAME.find(text_rest) {
                 let var_str = var_str.as_str();
                 let n_bytes = var_str.len();
                 cur_byte_offset += n_bytes;
@@ -167,23 +160,28 @@ struct PairPreCondition<'a, T: DataType> {
     apply: fn(&ParsedToken<'a, T>, &ParsedToken<'a, T>) -> ExResult<()>,
 }
 
-fn make_pair_pre_conditions<'a, 'b, T: DataType>() -> [PairPreCondition<'a, T>; 9] {
+fn make_err<T: DataType>(msg: &str, left: &ParsedToken<T>, right: &ParsedToken<T>) -> ExResult<()> {
+    Err(format_exerr!(
+        "{}, left: {:?}; right: {:?}",
+        msg,
+        left,
+        right
+    ))
+}
+
+fn make_pair_pre_conditions<'a, T: DataType>() -> [PairPreCondition<'a, T>; 9] {
     [
         PairPreCondition {
             apply: |left, right| {
                 let num_var_str =
                     "a number/variable cannot be next to a number/variable, violated by ";
                 match (left, right) {
-                    (ParsedToken::Num(num), ParsedToken::Var(var))
-                    | (ParsedToken::Var(var), ParsedToken::Num(num)) => Err(ExError {
-                        msg: format!("{}'{:?}' and '{}'", num_var_str, num, var),
-                    }),
-                    (ParsedToken::Num(num1), ParsedToken::Num(num2)) => Err(ExError {
-                        msg: format!("{}'{:?}' and '{:?}'", num_var_str, num1, num2),
-                    }),
-                    (ParsedToken::Var(var1), ParsedToken::Var(var2)) => Err(ExError {
-                        msg: format!("{}'{:?}' and '{:?}'", num_var_str, var1, var2),
-                    }),
+                    (ParsedToken::Num(_), ParsedToken::Var(_))
+                    | (ParsedToken::Var(_), ParsedToken::Num(_))
+                    | (ParsedToken::Num(_), ParsedToken::Num(_))
+                    | (ParsedToken::Var(_), ParsedToken::Var(_)) => {
+                        make_err(num_var_str, left, right)
+                    }
                     _ => Ok(()),
                 }
             },
@@ -193,10 +191,11 @@ fn make_pair_pre_conditions<'a, 'b, T: DataType>() -> [PairPreCondition<'a, T>;
                 (ParsedToken::Paren(_p @ Paren::Close), ParsedToken::Num(_))
                 | (ParsedToken::Paren(_p @ Paren::Close), ParsedToken::Var(_))
                 | (ParsedToken::Num(_), ParsedToken::Paren(_p @ Paren::Open))
-                | (ParsedToken::Var(_), ParsedToken::Paren(_p @ Paren::Open)) => Err(ExError {
-                    msg: "wlog a number/variable cannot be on the right of a closing parenthesis"
-                        .to_string(),
-                }),
+                | (ParsedToken::Var(_), ParsedToken::Paren(_p @ Paren::Open)) => make_err(
+                    "wlog a number/variable cannot be on the right of a closing parenthesis",
+                    left,
+                    right,
+                ),
                 _ => Ok(()),
             },
         },
@@ -205,12 +204,11 @@ fn make_pair_pre_conditions<'a, 'b, T: DataType>() -> [PairPreCondition<'a, T>;
                 (ParsedToken::Num(_), ParsedToken::Op(op))
                 | (ParsedToken::Var(_), ParsedToken::Op(op))
                     // we do not ask for is_unary since operators can be both
-                    if !op.has_bin() =>
-                {
-                    Err(ExError{
-                        msg: "a number/variable cannot be on the left of a unary operator".to_string()
-                    })
-                }
+                    if !op.has_bin() => make_err(
+                        "a number/variable cannot be on the left of a unary operator",
+                        left,
+                        right,
+                    ),                
                 _ => Ok(()),
             },
         },
@@ -218,16 +216,10 @@ fn make_pair_pre_conditions<'a, 'b, T: DataType>() -> [PairPreCondition<'a, T>;
             apply: |left, right| {
                 match (left, right) {
                 (ParsedToken::Op(op_l), ParsedToken::Op(op_r))
-                    if !op_l.has_unary() && !op_r.has_unary() =>
-                {
-                    Err(ExError {
-                        msg: format!(
-                            "a binary operator cannot be next to the binary operator, violated by '{}' left of '{}'",
-                            op_l.repr(),
-                            op_r.repr()
-                        ),
-                    })
-                }
+                    if !op_l.has_unary() && !op_r.has_unary() => Err(format_exerr!(
+                        "a binary operator cannot be next to the binary operator, violated by '{}' left of '{}'",
+                        op_l.repr(),
+                        op_r.repr())),                
                 _ => Ok(()),
             }
             },
@@ -236,56 +228,42 @@ fn make_pair_pre_conditions<'a, 'b, T: DataType>() -> [PairPreCondition<'a, T>;
             apply: |left, right| {
                 match (left, right) {
                 (ParsedToken::Op(op_l), ParsedToken::Op(op_r))
-                    if !op_l.has_bin() && !op_r.has_unary() =>
-                {
-                    Err(ExError {
-                        msg: format!(
-                            "a unary operator cannot be on the left of a binary one, violated by '{}' left of '{}'",
-                            op_l.repr(),
-                            op_r.repr()
-                        ),
-                    })
-                }
+                    if !op_l.has_bin() && !op_r.has_unary() => Err(format_exerr!(
+                        "a unary operator cannot be on the left of a binary one, violated by '{}' left of '{}'",
+                        op_l.repr(),
+                        op_r.repr())),                
                 _ => Ok(()),
             }
             },
         },
         PairPreCondition {
             apply: |left, right| match (left, right) {
-                (ParsedToken::Op(op), ParsedToken::Paren(_p @ Paren::Close)) => Err(ExError {
-                    msg: format!(
-                        "an operator cannot be on the left of a closing paren, violated by '{}'",
-                        op.repr()
-                    ),
-                }),
+                (ParsedToken::Op(op), ParsedToken::Paren(_p @ Paren::Close)) => Err(format_exerr!(
+                    "an operator cannot be on the left of a closing paren, violated by '{}'", op.repr())),                
                 _ => Ok(()),
             },
         },
         PairPreCondition {
             apply: |left, right| {
                 match (left, right) {
-                (ParsedToken::Paren(_p @ Paren::Close), ParsedToken::Op(op)) if !op.has_bin() => {
-                    Err(ExError{
-                        msg:format!(
-                            "a unary operator cannot be on the right of a closing paren, violated by '{}'", 
-                            op.repr()
-                        )
-                    })
+                    (ParsedToken::Paren(_p @ Paren::Close), ParsedToken::Op(op)) if !op.has_bin() => {
+                        Err(format_exerr!("a unary operator cannot be on the right of a closing paren, violated by '{}'", 
+                            op.repr()))
+                    }
+                    _ => Ok(()),
                 }
-                _ => Ok(()),
-            }
             },
         },
         PairPreCondition {
             apply: |left, right| {
                 match (left, right) {
-                (ParsedToken::Paren(_p @ Paren::Open), ParsedToken::Op(op)) if !op.has_unary() => {
-                    Err(ExError{
-                        msg:format!("a binary operator cannot be on the right of an opening paren, violated by '{}'", op.repr())
-                    })
+                    (ParsedToken::Paren(_p @ Paren::Open), ParsedToken::Op(op)) if !op.has_unary() => {
+                        Err(format_exerr!(
+                            "a binary operator cannot be on the right of an opening paren, violated by '{}'", 
+                            op.repr()))
+                    }
+                    _ => Ok(()),
                 }
-                _ => Ok(()),
-            }
             },
         },
         PairPreCondition {
@@ -293,9 +271,7 @@ fn make_pair_pre_conditions<'a, 'b, T: DataType>() -> [PairPreCondition<'a, T>;
                 (
                     ParsedToken::Paren(_p_l @ Paren::Open),
                     ParsedToken::Paren(_p_r @ Paren::Close),
-                ) => Err(ExError {
-                    msg: "wlog an opening paren cannot be next to a closing paren".to_string(),
-                }),
+                ) => make_err("wlog an opening paren cannot be next to a closing paren", left, right),                
                 _ => Ok(()),
             },
         },
@@ -312,11 +288,11 @@ fn make_pair_pre_conditions<'a, 'b, T: DataType>() -> [PairPreCondition<'a, T>;
 ///
 /// See [`parse_with_number_pattern`](parse_with_number_pattern)
 ///
-pub fn check_parsed_token_preconditions<'a, T>(parsed_tokens: &[ParsedToken<'a, T>]) -> ExResult<()>
+pub fn check_parsed_token_preconditions<T>(parsed_tokens: &[ParsedToken<T>]) -> ExResult<()>
 where
     T: DataType,
 {
-    if parsed_tokens.len() == 0 {
+    if parsed_tokens.is_empty() {
         return Err(ExError {
             msg: "cannot parse empty string".to_string(),
         });
@@ -372,9 +348,7 @@ where
 }
 
 #[cfg(test)]
-use {
-    crate::operators::{DefaultOpsFactory, MakeOperators}
-};
+use crate::operators::{FloatOpsFactory, MakeOperators};
 
 #[test]
 fn test_is_numeric() {
@@ -404,7 +378,7 @@ fn test_preconditions() {
             }
         }
 
-        let ops = DefaultOpsFactory::<f32>::make();
+        let ops = FloatOpsFactory::<f32>::make();
         let elts = tokenize_and_analyze(text, &ops, is_numeric_text);
         match elts {
             Err(e) => check_err_msg::<Vec<ParsedToken<f32>>>(Err(e), msg_part),
